#include "epolltcp.h"

#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/epoll.h>
#include <errno.h>


static int
make_socket_non_blocking (int sfd)
{
  int flags, s;

  flags = fcntl (sfd, F_GETFL, 0);
  if (flags == -1)
    {
      perror ("fcntl");
      return -1;
    }

  flags |= O_NONBLOCK;
  s = fcntl (sfd, F_SETFL, flags);
  if (s == -1)
    {
      perror ("fcntl");
      return -1;
    }

  return 0;
}

EpollTcp::EpollTcp(const std::string& /*address*/, const std::string& port, EpollInstance& e):EpollFd(-1, e) {
    
    struct addrinfo hints;
    struct addrinfo *result, *rp;
    int s, sfd;

    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_UNSPEC;     /* Return IPv4 and IPv6 choices */
    hints.ai_socktype = SOCK_STREAM; /* We want a TCP socket */
    hints.ai_flags = AI_PASSIVE;     /* All interfaces */
    
    s = getaddrinfo (NULL, port.c_str(), &hints, &result);
    if (s != 0) {
      fprintf (stderr, "getaddrinfo: %s\n", gai_strerror (s));
      exit(1);
    }

    for (rp = result; rp != NULL; rp = rp->ai_next) {
        sfd = socket (rp->ai_family, rp->ai_socktype | SOCK_NONBLOCK, rp->ai_protocol);
        if (sfd == -1) {
            continue;
        }

        s = bind (sfd, rp->ai_addr, rp->ai_addrlen);
        if (s == 0) {
            /* We managed to bind successfully! */
            break;
        }
        
        close (sfd);
    }
    
    
    if (rp == NULL) {
        fprintf (stderr, "Could not bind\n");
        exit(1);
    }

    freeaddrinfo(result);

    fd = sfd;
    // Make nonblocking and start listening
    s = make_socket_non_blocking(fd);
    if (s != 0) {
        fprintf(stderr, "failed making nonblocking");
    }
    s = listen(fd, SOMAXCONN);
    
    fprintf(stderr, "Registering tcpserv instance: %p\n", this);
    registerFd(EPOLLIN);
}


void EpollTcp::handleEvent(uint32_t events) {
    fprintf(stdout, "epoltcp handle event\n");
    if ((events & EPOLLERR) || (events & EPOLLHUP) || (!(events & EPOLLIN))) {
          /* An error has occured on this fd, or the socket is not
             ready for reading (why were we notified then?) */
	  fprintf (stderr, "EpollTcp error\n");
      unregisterFd();
      // ToDo:: Maybe close the fd?
	  // close (fd);
	} else {
        fprintf(stdout, "EpollTcp maybe connect\n");
        int cfd = accept4(fd, NULL, NULL, SOCK_NONBLOCK);
        // New connection is opened, lets register it as well
    }
}
